@@ -32,7 +32,7 @@
  * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
  */
 
-#define RCSID	"$Id: auth.c,v 1.4 2001/01/20 03:35:43 callie Exp $"
+#define RCSID	"$Id: auth.c,v 1.6 2001/06/05 23:57:22 callie Exp $"
 
 #include <stdio.h>
 #include <stddef.h>
@@ -132,6 +132,13 @@ void (*pap_logout_hook) __P((void)) = NULL;
 /* Hook for a plugin to get the PAP password for authenticating us */
 int (*pap_passwd_hook) __P((char *user, char *passwd)) = NULL;
 
+#ifdef __APPLE__
+struct notifier *auth_start_notify = NULL;
+struct notifier *auth_withpeer_fail_notify = NULL;
+struct notifier *auth_withpeer_success_notify = NULL;
+#endif
+
+
 /*
  * This is used to ensure that we don't start an auth-up/down
  * script while one is already running.
@@ -145,6 +152,8 @@ static enum script_state auth_state = s_down;
 static enum script_state auth_script_state = s_down;
 static pid_t auth_script_pid = 0;
 
+static int used_login;		/* peer authenticated against login database */
+
 /*
  * Option variables.
  */
@@ -190,6 +199,7 @@ static int  setupapfile __P((char **));
 static int  privgroup __P((char **));
 static int  set_noauth_addr __P((char **));
 static void check_access __P((FILE *, char *));
+static int  wordlist_count __P((struct wordlist *));
 
 /*
  * Authentication-related options.
@@ -229,14 +239,14 @@ option_t auth_options[] = {
       "Use system password database for PAP", 1 },
     { "papcrypt", o_bool, &cryptpap,
       "PAP passwords are encrypted", 1 },
-    { "+ua", o_special, setupapfile,
+    { "+ua", o_special, (void *)setupapfile,
       "Get PAP user and password from file" },
     { "password", o_string, passwd,
       "Password for authenticating us to the peer", OPT_STATIC,
       NULL, MAXSECRETLEN },
-    { "privgroup", o_special, privgroup,
+    { "privgroup", o_special, (void *)privgroup,
       "Allow group members to use privileged options", OPT_PRIV },
-    { "allow-ip", o_special, set_noauth_addr,
+    { "allow-ip", o_special, (void *)set_noauth_addr,
       "Set IP address(es) which can be used without authentication",
       OPT_PRIV },
     { NULL }
@@ -318,10 +328,10 @@ set_noauth_addr(argv)
     char **argv;
 {
     char *addr = *argv;
-    int l = strlen(addr);
+    int l = strlen(addr) + 1;
     struct wordlist *wp;
 
-    wp = (struct wordlist *) malloc(sizeof(struct wordlist) + l + 1);
+    wp = (struct wordlist *) malloc(sizeof(struct wordlist) + l);
     if (wp == NULL)
 	novm("allow-ip argument");
     wp->word = (char *) (wp + 1);
@@ -424,8 +434,8 @@ link_established(unit)
 	 * boot it out.
 	 */
 	if (noauth_addrs != NULL) {
-	    set_allowed_addrs(unit, noauth_addrs, NULL);
-	} else if (!wo->neg_upap || !null_login(unit)) {
+	    set_allowed_addrs(unit, NULL, NULL);
+	} else if (!wo->neg_upap || uselogin || !null_login(unit)) {
 	    warn("peer refused to authenticate: terminating link");
 	    lcp_close(unit, "peer refused to authenticate");
 	    status = EXIT_PEER_AUTH_FAILED;
@@ -434,6 +444,7 @@ link_established(unit)
     }
 
     new_phase(PHASE_AUTHENTICATE);
+    used_login = 0;
     auth = 0;
     if (go->neg_chap) {
 	ChapAuthPeer(unit, our_name, go->chap_mdtype);
@@ -456,6 +467,11 @@ link_established(unit)
     }
     auth_pending[unit] = auth;
 
+#ifdef __APPLE__
+    if (auth)
+        notify(auth_start_notify, auth);
+#endif
+
     if (!auth)
 	network_phase(unit);
 }
@@ -509,7 +525,18 @@ start_networks()
     struct protent *protp;
 
     new_phase(PHASE_NETWORK);
-#if 0
+
+#ifdef HAVE_MULTILINK
+    if (multilink) {
+	if (mp_join_bundle()) {
+	    if (updetach && !nodetach)
+		detach();
+	    return;
+	}
+    }
+#endif /* HAVE_MULTILINK */
+
+#ifdef PPP_FILTER
     if (!demand)
 	set_filters(&pass_filter, &active_filter);
 #endif
@@ -570,7 +597,7 @@ auth_peer_success(unit, protocol, name, namelen)
 	namelen = sizeof(peer_authname) - 1;
     BCOPY(name, peer_authname, namelen);
     peer_authname[namelen] = 0;
-    script_setenv("PEERNAME", peer_authname);
+    script_setenv("PEERNAME", peer_authname, 0);
 
     /*
      * If there is no more authentication still to be done,
@@ -588,6 +615,10 @@ auth_withpeer_fail(unit, protocol)
     int unit, protocol;
 {
 
+#ifdef __APPLE__
+    notify(auth_withpeer_fail_notify, protocol);
+#endif
+
     if (passwd_from_file)
 	BZERO(passwd, MAXSECRETLEN);
     /*
@@ -609,6 +640,10 @@ auth_withpeer_success(unit, protocol)
 {
     int bit;
 
+#ifdef __APPLE__
+    notify(auth_withpeer_success_notify, protocol);
+#endif
+
     switch (protocol) {
     case PPP_CHAP:
 	bit = CHAP_WITHPEER;
@@ -905,21 +940,29 @@ check_passwd(unit, auser, userlen, apasswd, passwdlen, msg)
 	check_access(f, filename);
 	if (scan_authfile(f, user, our_name, secret, &addrs, &opts, filename) < 0) {
 	    warn("no PAP secret found for %s", user);
-	} else if (secret[0] != 0) {
-	    /* password given in pap-secrets - must match */
-	    if ((!cryptpap && strcmp(passwd, secret) == 0)
-		|| strcmp(crypt(passwd, secret), secret) == 0)
-		ret = UPAP_AUTHACK;
-	    else
-		warn("PAP authentication failure for %s", user);
-	} else if (uselogin) {
-	    /* empty password in pap-secrets and login option */
-	    ret = plogin(user, passwd, msg);
-	    if (ret == UPAP_AUTHNAK)
-		warn("PAP login failure for %s", user);
 	} else {
-	    /* empty password in pap-secrets and login option not used */
+	    /*
+	     * If the secret is "@login", it means to check
+	     * the password against the login database.
+	     */
+	    int login_secret = strcmp(secret, "@login") == 0;
 	    ret = UPAP_AUTHACK;
+	    if (uselogin || login_secret) {
+		/* login option or secret is @login */
+		ret = plogin(user, passwd, msg);
+		if (ret == UPAP_AUTHNAK)
+		    warn("PAP login failure for %s", user);
+		else
+		    used_login = 1;
+	    }
+	    if (secret[0] != 0 && !login_secret) {
+		/* password given in pap-secrets - must match */
+		if ((cryptpap || strcmp(passwd, secret) != 0)
+		    && strcmp(crypt(passwd, secret), secret) != 0) {
+		    ret = UPAP_AUTHNAK;
+		    warn("PAP authentication failure for %s", user);
+		}
+	    }
 	}
 	fclose(f);
     }
@@ -1368,6 +1411,7 @@ get_secret(unit, client, server, secret, secret_len, am_server)
     if (!am_server && passwd[0] != 0) {
 	strlcpy(secbuf, passwd, sizeof(secbuf));
     } else {
+
 	filename = _PATH_CHAPFILE;
 	addrs = NULL;
 	secbuf[0] = 0;
@@ -1416,7 +1460,7 @@ set_allowed_addrs(unit, addrs, opts)
     struct wordlist *opts;
 {
     int n;
-    struct wordlist *ap, **pap;
+    struct wordlist *ap, **plink;
     struct permitted_ip *ip;
     char *ptr_word, *ptr_mask;
     struct hostent *hp;
@@ -1435,14 +1479,18 @@ set_allowed_addrs(unit, addrs, opts)
     /*
      * Count the number of IP addresses given.
      */
-    for (n = 0, pap = &addrs; (ap = *pap) != NULL; pap = &ap->next)
-	++n;
+    n = wordlist_count(addrs) + wordlist_count(noauth_addrs);
     if (n == 0)
 	return;
     ip = (struct permitted_ip *) malloc((n + 1) * sizeof(struct permitted_ip));
     if (ip == 0)
 	return;
 
+    /* temporarily append the noauth_addrs list to addrs */
+    for (plink = &addrs; *plink != NULL; plink = &(*plink)->next)
+	;
+    *plink = noauth_addrs;
+
     n = 0;
     for (ap = addrs; ap != NULL; ap = ap->next) {
 	/* "-" means no addresses authorized, "*" means any address allowed */
@@ -1532,6 +1580,7 @@ set_allowed_addrs(unit, addrs, opts)
 	if (~mask == 0 && suggested_ip == 0)
 	    suggested_ip = a;
     }
+    *plink = NULL;
 
     ip[n].permit = 0;		/* make the last entry forbid all addresses */
     ip[n].base = 0;		/* to terminate the list */
@@ -1571,7 +1620,7 @@ auth_ip_addr(unit, addr)
     }
     if (auth_required)
 	return 0;		/* no addresses authorized */
-    return allow_any_ip || !have_route_to(addr);
+    return allow_any_ip || privileged || !have_route_to(addr);
 }
 
 static int
@@ -1721,25 +1770,26 @@ scan_authfile(f, client, server, secret, addrs, opts, filename)
 	if (newline)
 	    continue;
 
-	/*
-	 * Special syntax: @filename means read secret from file.
-	 */
-	if (word[0] == '@') {
-	    strlcpy(atfile, word+1, sizeof(atfile));
-	    if ((sf = fopen(atfile, "r")) == NULL) {
-		warn("can't open indirect secret file %s", atfile);
-		continue;
-	    }
-	    check_access(sf, atfile);
-	    if (!getword(sf, word, &xxx, atfile)) {
-		warn("no secret in indirect secret file %s", atfile);
+	if (secret != NULL) {
+	    /*
+	     * Special syntax: @/pathname means read secret from file.
+	     */
+	    if (word[0] == '@' && word[1] == '/') {
+		strlcpy(atfile, word+1, sizeof(atfile));
+		if ((sf = fopen(atfile, "r")) == NULL) {
+		    warn("can't open indirect secret file %s", atfile);
+		    continue;
+		}
+		check_access(sf, atfile);
+		if (!getword(sf, word, &xxx, atfile)) {
+		    warn("no secret in indirect secret file %s", atfile);
+		    fclose(sf);
+		    continue;
+		}
 		fclose(sf);
-		continue;
 	    }
-	    fclose(sf);
-	}
-	if (secret != NULL)
 	    strlcpy(lsecret, word, sizeof(lsecret));
+	}
 
 	/*
 	 * Now read address authorization info and make a wordlist.
@@ -1795,6 +1845,20 @@ scan_authfile(f, client, server, secret, addrs, opts, filename)
     return best_flag;
 }
 
+/*
+ * wordlist_count - return the number of items in a wordlist
+ */
+static int
+wordlist_count(wp)
+    struct wordlist *wp;
+{
+    int n;
+
+    for (n = 0; wp != NULL; wp = wp->next)
+	++n;
+    return n;
+}
+
 /*
  * free_wordlist - release memory allocated for a wordlist.
  */
